--- This module provides an abstraction layer for the board.
-- The following methods are public:
-- board.setVideoOutput()
-- board.getVideoOutputConfig(port)
-- board.setVideoInput()
-- board.getVideoInputConfig(port)
-- board.getType()
-- board.getHost()
--
-- board specific settings (gpios, etc) must be defined in the board._support
-- table. This table is indexed by board key (board type + host), io type (e.g. 
-- video input, etc.) and finally io mode (e.g. svideo)
--
-- Any errors in the board object results in a hard assert. This is to ensure 
-- that the board is properly configured prior to starting any script.
-- If for some reason, assertions are to be ignored, call the methods using 
-- pcall()
--
-- local code, err = pcall(board.setVideoOutput, {port = 0, mode = hdmi, ...}}
--
-- Testing Modifications to the Board Object
-- -----------------------------------------
--
-- The 40board.lua is normally compiled with mobiapp. A way to test this script
-- without recompiling mobiapp is to start mobiapp, and then explicitely load 
-- it from the mobiapp interpreter using dofile("40board.lua").
-- -----------------------------------------------------------------------------

-- -----------------------------------------------------------------------------
-- @(#) $Id: 40board.lua,v 1.4 2011-03-23 22:45:02 mlee Exp $
-- -----------------------------------------------------------------------------

-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
-- PUT CODE AFTER THE DO BELOW 
-- DO NOT REMOVE the do statement nor its matching end (end of the board object 
-- file). If you have any question about this, ask Laurent Brack
-->>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

do

local board         = {}
board._ifo          = {}
--------------------------------------------------------------------------------
-- Initialization. All private data is stored in board._data.
-- _data.vout, _data.vin are indexed by port number and contains that last 
-- configuration parameters set by the caller.
board._data         = {}
board._data.sys_rc  = 0 -- return code expected from system call. This can be 
                        -- changed before calling the board.execute() function.
                        -- for instance qccrw return 1 on success and 0 on error.
board._data.vout    = {}
board._data.vin     = {}
board._data.ain     = {}
board._data.audio   = {}
board._config       = {} -- used to maintain setting states


--------------------------------------------------------------------------------
-- This table contains board specific calls to configure IOs.
-- The first level is the board/host identifier which is computed when the module is loaded.
-- The second level is the IO type and finally, the third level is the mode.
-- @class table
-- @name board._support 
-- @field mips_merlinbup        Merlin Bringup (coproc)
-- @field mg3500_merlinbup      Merlin Bringup SOC mode (not working)
-- @field mg3500_evp            Evalution Platform
-- @field i686_workstation      reserved for QA use
-- @field mobicam2              mobicam2
-- @field mobicam3              mobicam3
--
-- @see board.detect
board._support = 
{
   -----------------------------------------------------------------------------
   -- Merlin BUP Mips
   mips_merlinbup = {
      dev_root   = "/home/merlin/bin/Linux.mips", -- used in board.set_device
      gpio_root  = "/proc/driver/gpio/sbus_gpio", -- require by board.wr_gpio
      sysctlvout = function(ifo)
          if ifo.format == "720p" or ifo.format == "1080i" then
             if ifo.clockntsc == 0 then
               sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value = 74250000})
               sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value = 74250000})
             else
               sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value = 74175824})
               sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value = 74175824})
             end

             sysctl0:configure({param=Q_SYS_CFG_VIDEO_INVERT_CONTROL,     value = 0x00000004})
             sysctl0:configure({param=Q_SYS_CFG_SERIALIO_DRIVESTRENGTH_2, value = 0x0000277b})   
             sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL,             value = 0x0012aac0})
             
          else
             sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value = 27000000})
             sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value = 27000000})
             sysctl0:configure({param=Q_SYS_CFG_VIDEO_INVERT_CONTROL,     value = 0x00000004})
             sysctl0:configure({param=Q_SYS_CFG_SERIALIO_DRIVESTRENGTH_2, value = 0x00002539})                  
             sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL,             value = 0x0012aa00})      
          end             
      end, 
      sysctlvin = function(ifo)
          if ifo.format == "720p" or ifo.format == "1080i" then
             sysctl0:configure({param=Q_SYS_CFG_VIN0_CONTROL,             value = 0x00048249}) 
             sysctl0:configure({param=Q_SYS_CFG_VIN1_CONTROL,             value = 0x00000000})
          else   
             if ifo.port == 1 then
                 sysctl0:configure({param=Q_SYS_CFG_VIN1_CONTROL,             value = 0x00049249})
             else
                 sysctl0:configure({param=Q_SYS_CFG_VIN0_CONTROL,             value = 0x00040000})
             end
          end
       end,
      --------------------------------------------------------------------------
      -- AUDIO INPUT
      ain = {
              --===============================================================
              onboard = function(ifo)
			              board.execute("i2crw /dev/i2c 0x20 0x0 0xc")
			              local sample = ifo.freq * 256;
			              local oversample = ifo.freq * 64;
			              print("Audio clock settings: " .. sample .. " " .. oversample);
			              sysctl0:configure({param=Q_SYS_CFG_PLL3_CLOCK_HZ, value=sample});
			              if ifo.port == 0 then
			                 sysctl0:configure({param=Q_SYS_CFG_AUD0_CLOCK_HZ, value=sample})
			              else -- check for port ID is made in the set function
			                 sysctl0:configure({param=Q_SYS_CFG_AUD1_CLOCK_HZ, value=sample})
			              end
			              print("Audio clock settings set");
                        end,
       },       
      -- END AUDIO OUTPUT
      --------------------------------------------------------------------------                      
      --------------------------------------------------------------------------
      -- VIDEO OUTPUT
      vout = {
               --===============================================================
               composite   = function(ifo)
                            board._support.mips_merlinbup.vout.svideo(ifo)
                        end,
               --===============================================================

               svideo = function(ifo)
                           -- change the fpga switch to route to vo_01 board
                           board.wr_gpio("18", 1)
                           board.wr_gpio("17", 0)
                           board.wr_gpio("16", 0)
                           -- reset the video encoder
                           board.wr_gpio("00", 1)
                           -- wait a bit
                           board.syscall("usleep 100000");
                           -- take it out of reset
                           board.wr_gpio("00", 0)
                           -- program the encoder
                           if     ifo.format == "480i" then
                                board.set_device("SAA7121", "--ntsc")
                           elseif ifo.format == "576i" then
                                board.set_device("SAA7121", "--pal")
                           else
                               error(tostring(ifo.format) .. " format not supported on svideo") 
                           end    
                        end,
               --===============================================================         
               hdmi   = function(ifo)
                           if ifo.external == 1 then
                             board.set_device("AD9889", "--dev 0x7A --format %s --input YUV --output RGB --style 2 --noDE", ifo.format)
                           else
                             board.set_device("AD9889", "--dev 0x7A --format %s --input YUV --output RGB --style 2 --656", ifo.format)
                           end
                           board.wr_gpio("19", 1)
                        end,
               --===============================================================
               sdi    = function(ifo)
                           -- resetting the sdi output chip 
                           board.wr_gpio("02", 1)
                           board.syscall("sleep 1")
                           -- removing reset the output 
                           board.wr_gpio("02", 0)

                          if ifo.format == "720p" or ifo.format == "1080i" then
                             -- setting up the mode for sdi out
                             -- hd mode 1:113 0:114
                             board.wr_gpio("113", 1)
                             board.wr_gpio("114", 0)
                           else
                             -- sd mode 0:113 1:114
                             board.wr_gpio("113", 0)
                             board.wr_gpio("114", 1)
                           end
                           board.wr_gpio("20", 1)
                         end
      }, 
      -- END VIDEO OUTPUT
      --------------------------------------------------------------------------
      -- VIDEO INPUT          
      vin = {
               --===============================================================
               component = function(ifo)
                          --component input
                          board.set_device("AD9880", "--format %s --input YUV --output YUV --analog 0", ifo.format)
                          -- The VID 0,1 are getting HDMI in 
                          board.execute("echo $((0x60)) $((0x40)) >> /proc/driver/gpio/sbus_gpio/bank1"); 
                       end,
               --===============================================================
               svideo = function(ifo)
                          if ifo.port == 1 then
                                 reset_gpio = "04"
                                 address = "0x40"
                          else   
                                 reset_gpio = "03"
                                 address = "0x42"
                          end
                          board.wr_gpio(reset_gpio, 1)
                          -- wait a bit
                          board.syscall("usleep 100000")
                          -- take it out of reset
                          board.wr_gpio(reset_gpio, 0)
                          -- program the decoder

                          if     ifo.format == "480i" then
                                board.set_device("SAA7114", "--ntsc_svideo --dev "..address)
                          elseif ifo.format == "576i" then
                                board.set_device("SAA7114", "--pal_svideo --dev "..address)
                          else
                               error(tostring(ifo.format) .. " format not supported on svideo") 
                          end                          
                          board.wr_gpio("21", 1)
                          board.wr_gpio("22", 0)

                      end,

               --===============================================================
               composite = function(ifo)
                          if ifo.port == 1 then
                                 reset_gpio = "04"
                                 address = "0x40"
                          else
                                 reset_gpio = "03"
                                 address = "0x42"
                          end
                          board.wr_gpio(reset_gpio, 1)
                          -- wait a bit
                          board.syscall("usleep 100000")
                          -- take it out of reset
                          board.wr_gpio(reset_gpio, 0)
                          -- program the decoder
                          if     ifo.format == "480i" then
                                board.set_device("SAA7114", "--ntsc_composite --dev "..address)
                          elseif ifo.format == "576i" then
                                board.set_device("SAA7114", "--pal_composite --dev "..address)
                          else
                               error(tostring(ifo.format) .. " format not supported on svideo")
                          end
                          -- route the inputs (assume both ports for now)

                          board.wr_gpio("21", 1)
                          board.wr_gpio("22", 0)
                      end,
               --==MGCLI_FRAME_RATE_PAL=============================================================
               hdmi = function(ifo)
                          --HDMI in 
                          board.set_device("AD9880", "--format %s --input RGB --output YUV --digital", ifo.format)
                          -- The VID 0,1 are getting HDMI in 
                          board.execute("echo $((0x60)) $((0x40)) >> /proc/driver/gpio/sbus_gpio/bank1");
                          --board.wr_gpio(03, 1)                
                      end,
               --===============================================================
               sdi  = function(ifo)
                          -- ### resetting the sdi input chip 
                          board.wr_gpio("05", 1)    
                          board.syscall("sleep 1")

                          --### removing reset the input 
                          board.wr_gpio("05", 0)
                          if ifo.format == "720p" or ifo.format == "1080i" then
                             -- setting up the mode for sdi in
                             -- setting up the 16 bit mode for sdi input
                             -- 1 == 16 bit
                             board.wr_gpio("115", 1)
                          else
                             -- setting up the mode for sdi in
                             -- setting up the 8 bit mode for sdi input
                             -- 0 == 8 bit
                             board.wr_gpio("115", 0)                             
                          end
                          --SDI to Vid 0, 1
                          board.wr_gpio("21", 1)
                          board.wr_gpio("22", 1)
                        end
               -----------------------------------------------------------------
      }, 
      -- END VIDEO INPUT
      --------------------------------------------------------------------------
   }, 
   -- END Merlin BUP Mips
    -----------------------------------------------------------------------------
    -- Merlin BUP SOC
    mg3500_merlinbup = {
        dev_root   = "/usr/bin", -- used in board.set_device
        gpio_root  = "/proc/driver/gpio/native", -- require by board.wr_gpio
        sysctlvout = function(ifo)
            if ifo.format == "720p" or ifo.format == "1080i" then
                    if ifo.clockntsc == 0 then
                      sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=74250000})
                      sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value=74250000})
                    else
                      sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=74175824})
                      sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value=74175824})
                    end

                    sysctl0:configure({param=Q_SYS_CFG_VIDEO_INVERT_CONTROL,     value=0x00000004})
                    sysctl0:configure({param=Q_SYS_CFG_SERIALIO_DRIVESTRENGTH_2, value=0x0000277b})   
                    sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL,             value=0x0012aac0})             
                else
                    sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=27000000})
                    sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value=27000000})
                    sysctl0:configure({param=Q_SYS_CFG_VIDEO_INVERT_CONTROL,     value=0x00000004})
                    sysctl0:configure({param=Q_SYS_CFG_SERIALIO_DRIVESTRENGTH_2, value=0x00002539})   
                    sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL,             value=0x0012aa00})      
                end    
            end,
                   
        sysctlvin = function(ifo)
            if ifo.format == "720p" or ifo.format == "1080i" then
                sysctl0:configure({param=Q_SYS_CFG_VIN0_CONTROL, value=0x00048249}) 
                sysctl0:configure({param=Q_SYS_CFG_VIN1_CONTROL, value=0x00000000})
            else
                sysctl0:configure({param=Q_SYS_CFG_VIN0_CONTROL, value=0x00040000})
                sysctl0:configure({param=Q_SYS_CFG_VIN1_CONTROL, value=0x00000000})
            end
        end,                   
		
	

        --------------------------------------------------------------------------
        -- VIDEO OUTPUT
        vout = {
            --===============================================================
            component = 
                function(ifo)
                    error("Component not supported") 
                end,
            --===============================================================
            svideo =  
                function(ifo)                           
                    if ifo.format == "480i" then
                        board.set_device("SAA7121", "--bus /dev/i2c-0 --dev 0x88 --ntsc")
                    elseif ifo.format == "576i" then
                        board.set_device("SAA7121", "--bus /dev/i2c-0 --dev 0x88 --pal")
                    else
                        error(tostring(ifo.format) .. " format not supported on svideo") 
                    end                          

                end,
            --===============================================================         
            --===============================================================
            composite =  
                function(ifo)                           
                    if ifo.format == "480i" then
                        board.set_device("SAA7121", "--bus /dev/i2c-0 --dev 0x88 --ntsc")
                    elseif ifo.format == "576i" then
                        board.set_device("SAA7121", "--bus /dev/i2c-0 --dev 0x88 --pal")
                    else
                        error(tostring(ifo.format) .. " format not supported on composite") 
                    end                          

                end,
            --===============================================================         
            hdmi = 
                function(ifo)
                    if ifo.external == 1 then
                        board.set_device("AD9889", 
                                "--bus /dev/i2c-0 --dev 0x7A --format %s --input YUV --output RGB --style 2 --noDE", 
                                ifo.format)
                    else
                        board.set_device("AD9889", 
                                "--bus /dev/i2c-0 --dev 0x7A --format %s --input YUV --output RGB --style 2 --656", 
                                ifo.format)
                    end
                end,
            --===============================================================
            sdi = 
                function(ifo)
                    -- 0=8bits, 1=16bits
                    board.wr_gpio("02", 1)
                    if ifo.format == "720p" or ifo.format == "1080i" then
                        board.wr_gpio("04", 0)
                    else
                        board.wr_gpio("04", 1)
                    end
                end,
            --===============================================================         
        }, 
        -- END VIDEO OUTPUT
        --------------------------------------------------------------------------
        -- VIDEO INPUT          
        vin = {
            --===============================================================
            component = 
                function(ifo)
                    board.wr_gpio("03", 1)
                    -- Enable a clock, necessary because of a bug in SAA7121 that may otherwise hang I2C
                    board.execute("echo 8 0x58 4 0x24003ff7 > /proc/driver/qcc/writebid");
                    -- Set input switch in FPGA
                    board.wr_gpio("01", 1)
                    board.wr_gpio("00", 0)
                    board.syscall("usleep 100000")
                    board.wr_gpio("03", 0)
                    board.syscall("usleep 100000")

                    --component input
                    board.set_device("AD9880", "--format %s --input YUV --output YUV --analog 0", ifo.format)
                end,
            --===============================================================
            svideo =    
                function(ifo)                          
                    board.wr_gpio("03", 1)
                    -- Enable a clock, necessary because of a bug in SAA7121 that may otherwise hang I2C
                    board.execute("echo 8 0x58 4 0x24003ff7 > /proc/driver/qcc/writebid");
                    -- Set input switch in FPGA
                    board.wr_gpio("01", 0)
                    board.wr_gpio("00", 1)
                    board.syscall("usleep 100000")
                    board.wr_gpio("03", 0)
                    board.syscall("usleep 100000")

                    if ifo.port == 1 then
                        address = "0x40"
                    else
                        address = "0x42"
                    end
                    if ifo.format == "480i" then
                        board.set_device("SAA7114", "--bus /dev/i2c-0 --dev "..address.." --ntsc_svideo");
                    elseif ifo.format == "576i" then
                        board.set_device("SAA7114", "--bus /dev/i2c-0 --dev "..address.." --pal_svideo");
                    else
                        error(tostring(ifo.format) .. " format not supported on svideo") 
                    end                          

                end,
            --===============================================================
            composite =    
                function(ifo)
                    board.wr_gpio("03", 1)
                    -- Enable a clock, necessary because of a bug in SAA7121 that may otherwise hang I2C
                    board.execute("echo 8 0x58 4 0x24003ff7 > /proc/driver/qcc/writebid");
                    -- Set input switch in FPGA
                    board.wr_gpio("01", 0)
                    board.wr_gpio("00", 1)
                    board.syscall("usleep 100000")
                    board.wr_gpio("03", 0)
                    board.syscall("usleep 100000")
               
                    if ifo.port == 1 then
                        address = "0x40"
                    else
                        address = "0x42"
                    end
                    if ifo.format == "480i" then
                        board.set_device("SAA7114", "--bus /dev/i2c-0 --dev "..address.." --ntsc_composite");
                    elseif ifo.format == "576i" then
                        board.set_device("SAA7114", "--bus /dev/i2c-0 --dev "..address.." --pal_composite");
                    else
                        error(tostring(ifo.format) .. " format not supported on component") 
                    end                          
                end,
            --===============================================================
            hdmi = 
                function(ifo)
                    board.wr_gpio("03", 1)
                    -- Enable a clock, necessary because of a bug in SAA7121 that may otherwise hang I2C
                    board.execute("echo 8 0x58 4 0x24003ff7 > /proc/driver/qcc/writebid");
                    -- Set input switch in FPGA
                    board.wr_gpio("01", 1)
                    board.wr_gpio("00", 0)
                    board.syscall("usleep 100000")
                    board.wr_gpio("03", 0)
                    board.syscall("usleep 100000")

                    board.set_device("AD9880", "--format %s --input RGB --output YUV --digital", ifo.format)
                end,
            --===============================================================
            sdi = 
                function(ifo)
                    board.wr_gpio("03", 1)
                    -- Enable a clock, necessary because of a bug in SAA7121 that may otherwise hang I2C
                    board.execute("echo 8 0x58 4 0x24003ff7 > /proc/driver/qcc/writebid");
                    -- Set input switch in FPGA
                    board.wr_gpio("01", 1)
                    board.wr_gpio("00", 1)
                    board.syscall("usleep 100000")
                    board.wr_gpio("03", 0)
                    board.syscall("usleep 100000")
                    if ifo.format == "720p" or ifo.format == "1080i" then
                        -- 16 bit
                        board.wr_gpio("05", 1)
                    else
                        -- 8 bit
                        board.wr_gpio("05", 0)
                    end
                end,
            -----------------------------------------------------------------
        }, 
        -- END VIDEO INPUT
        -------------------------------------------------------------------------- 
       --------------------------------------------------------------------------
      -- AUDIO INPUT
      ain = {
              --===============================================================
              onboard = function(ifo)
			              board.execute("i2crw /dev/i2c 0x20 0x0 0xc")
			              local sample = ifo.freq * 256;
			              local oversample = ifo.freq * 64;
			              print("Audio clock settings: " .. sample .. " " .. oversample);
			              sysctl0:configure({param=Q_SYS_CFG_PLL3_CLOCK_HZ, value=sample});
			              if ifo.port == 0 then
			                 sysctl0:configure({param=Q_SYS_CFG_AUD0_CLOCK_HZ, value=sample})
			              else -- check for port ID is made in the set function
			                 sysctl0:configure({param=Q_SYS_CFG_AUD1_CLOCK_HZ, value=sample})
			              end
			              print("Audio clock settings set");
                        end,
       },       
      -- END AUDIO OUTPUT
      --------------------------------------------------------------------------  
                     
    },
    -- END Merlin BUP SOC

    -----------------------------------------------------------------------------
    -- ARM MG3500 Proto
    mg3500_merlinproto = {
      --------------------------------------------------------------------------
      -- AUDIO OUTPUT
       ain = {
              --===============================================================
              onboard = function(ifo)
                        end,
       },       
      -- END AUDIO OUTPUT
      --------------------------------------------------------------------------
      
      --------------------------------------------------------------------------
      -- VIDEO OUTPUT
       vout = {
                --===============================================================
                component = function(ifo)
                            end,
                --===============================================================
                svideo =    function(ifo)                           
                            end,
                --===============================================================         
                hdmi      = function(ifo)
                            end,
                --===============================================================
                sdi     =   function(ifo)
                            end
       }, 
       -- END VIDEO OUTPUT
       --------------------------------------------------------------------------
       -- VIDEO INPUT          
       vin = {
                --===============================================================
                component = function(ifo)
                            end,
                --===============================================================
                svideo =    function(ifo)
                            end,
                --===============================================================
                hdmi      = function(ifo)
                            end,
                --===============================================================
                sdi   =     function(ifo)
                            end
                -----------------------------------------------------------------
       }, 
       -- END VIDEO INPUT -------------------------------------------------------
              
    },
    -- END ARM MG3500 Proto

   -----------------------------------------------------------------------------
   -- Merlin Proto
   mips_merlinproto = {
      --------------------------------------------------------------------------
      -- AUDIO OUTPUT
       ain = {
              --===============================================================
              onboard = function(ifo)
                        end,
       },       
      -- END AUDIO OUTPUT
      --------------------------------------------------------------------------
      
      --------------------------------------------------------------------------
      -- VIDEO OUTPUT
      vout = {
               --===============================================================
               component = function(ifo)
                           end,
               --===============================================================
               svideo =    function(ifo)                           
                           end,
               --===============================================================         
               hdmi      = function(ifo)
                           end,
               --===============================================================
               sdi     =   function(ifo)
                           end
      }, 
      -- END VIDEO OUTPUT
      --------------------------------------------------------------------------
         -- VIDEO INPUT          
      vin = {
               --===============================================================
               component = function(ifo)
                           end,
               --===============================================================
               svideo =    function(ifo)
                           end,
               --===============================================================
               hdmi      = function(ifo)
                           end,
               --===============================================================
               sdi   =     function(ifo)
                           end
               -----------------------------------------------------------------
      }, 
      -- END VIDEO INPUT
      --------------------------------------------------------------------------
             
   },
   -- END Merlin Proto
   -----------------------------------------------------------------------------
   -- EVP SOC
    mg3500_evp2 = {
        dev_root   = "/usr/bin", -- used in board.set_device
        gpio_root  = "/proc/driver/gpio/native", -- require by board.wr_gpio
        sysctlvout = function(ifo)
          if ifo.format == "720p" or ifo.format == "1080i"  or ifo.format == "720p50"  or ifo.format == "1080i50" then
             if ifo.clockntsc == 0 then
               sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=74250000})
               sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value=74250000})
             else
               sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=74175824})
               sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value=74175824})
             end

             sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL,             value=0x0012aac0})             
          else
             sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=27000000})
             sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value=27000000})
             sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL,             value=0x0012aa00})      
          end    
          sysctl0:configure({param=Q_SYS_CFG_VIDEO_INVERT_CONTROL,     value=0x00000000})
        end,
                   
        sysctlvin = function(ifo)
          if ifo.format == "720p" or ifo.format == "1080i" or ifo.format == "720p50" or ifo.format == "1080i50" or ifo.mode == "kite" then
             sysctl0:configure({param=Q_SYS_CFG_VIN0_CONTROL, value=0x00048000}) 
             sysctl0:configure({param=Q_SYS_CFG_VIN1_CONTROL, value=0x00000000})
          else
             if ifo.port == 1 then
                 sysctl0:configure({param=Q_SYS_CFG_VIN1_CONTROL,             value = 0x00049249})
             else
                 sysctl0:configure({param=Q_SYS_CFG_VIN0_CONTROL,             value = 0x00040000})
             end
          end
        end,

       -- boardInit is called by the detect() method once the board has been detected.
       boardInit = function(ifo)
        local h = assert(io.open(board.cfg.gpio_root, "r", "Failed to find gpio path"))
        h:close()
             
	    board.wr_gpio("46", 1)
	    board.wr_gpio("46", 0)
	    board.syscall("usleep 100000")
	    board.wr_gpio("46", 1)
      end,
     
             
      -- Kite reset shall be called the first time the board is initialized
      -- Enabling 54MHz clock to Kite does not do any harm even if there is an 
      -- oscillator on Kite board. 
      -- Since the clock is enabled there and reset pulse is already toggled, you
      -- do not need to toggle it again. 
      kiteReset = function()
      
        if board._config.kite_reset ~= true then
        
            -- This should be done only once
      
	        sysctl0:configure   {
	                                 param   =   Q_SYS_CFG_V0_PIXEL_CLOCK_FLAGS, 
	                                 value   =   (Q_SYS_CFP_CLOCK_FLAG_OUTPUT + Q_SYS_CFP_CLOCK_FLAG_INPUT)
	                            }
	
	        sysctl0:configure   {
	                                 param   =   Q_SYS_CFG_V0_PIXEL_CLOCK_HZ, 
	                                 value   =   54000000
	                            }
	        sysctl0:configure   {
	                                 param   =   Q_SYS_CFG_SERIALIO_DRIVESTRENGTH_1, 
	                                 value   =   0x000026c9
	                            }
	                    
	        board.wr_gpio("46", 1)
	        board.wr_gpio("46", 0)
	        board.syscall("usleep 100000")
	        board.wr_gpio("46", 1)
	        board._config.kite_reset = true
	        
        end      
      
      end,
      
        --------------------------------------------------------------------------
        -- AUDIO INPUT
        -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 7")
        -- 0: audio0 from Kite
        -- 1: audio0 from EVP on board
        -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 9")
        -- 0: audio1 from Kite
        -- 1: audio1 from EVP on board
        -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 0")
        -- 0: MG3500 becomes audio output clock slave (Kite)
        -- 1: MG3500 becomes audio output clock master (EVP)       
        
        ain = {
        
            kite = function(ifo)
            	print("kite ifo function\n");
                -- The following is only executed once
                board.cfg.kiteReset()

                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 0")                
                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 7")
                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 9")
                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 0")
                
                -- Techwell 2864
                -- The audio related TW register settings are the following:

                -- set 4-ch I2S output
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0xd2 0x01")
		
                -- map audio 2 and 3 to the second half of IIS frame
				board.execute("i2crw --sccb /dev/i2c-2 0x50 0xd3 0x10")
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0xd7 0x32")

                -- fixes for audio suggested by TW:
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0xd0 0x88")
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0xd1 0x88")
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0x82 0x00")
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0xce 0x40")
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0xe0 0x10")

				-- ACKG loop to be open
				board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf8 0x64")

				-- set output gain to be 1.00
				board.execute("i2crw --sccb /dev/i2c-2 0x50 0xdf 0x88")
				
				-- MISC control 
				board.execute("i2crw --sccb /dev/i2c-2 0x50 0x96 0xe6") 
				board.execute("i2crw --sccb /dev/i2c-2 0x50 0x9e 0x72")

				-- 2 channel multiplexed
				board.execute("i2crw --sccb /dev/i2c-2 0x50 0xca 0x55")

				-- channel selection
				board.execute("i2crw --sccb /dev/i2c-2 0x50 0xcc 0xee")
				board.execute("i2crw --sccb /dev/i2c-2 0x50 0xcd 0x44")

				-- clock setting
				board.execute("i2crw --sccb /dev/i2c-2 0x50 0xfb 0x0f")
				board.execute("i2crw --sccb /dev/i2c-2 0x50 0xfa 0x40")
				                
				if(ifo.format == nil) then
					ifo.format = "480i";
				end
		-- set up audio clock frequency
		-- 8kHz - NTSC
		if (ifo.freq == 8000 and ifo.format == "480i") then 
			print("format selected\n");
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf0 0x83")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf1 0xb5")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf2 0x09")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf3 0x78")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf4 0x85")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf5 0x00")
		--16 kHz - NTSC
		elseif(ifo.freq == 16000 and ifo.format == "480i") then
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf0 0x07")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf1 0x6b")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf2 0x13")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf3 0xef")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf4 0x0a")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf5 0x01")
		-- 32kHz - NTSC
		elseif(ifo.freq == 32000 and ifo.format == "480i") then
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf0 0x0e")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf1 0xd6")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf2 0x26")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf3 0xde")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf4 0x15")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf5 0x02")
		-- 8kHz - PAL
		elseif(ifo.freq == 8000 and ifo.format == "576i") then
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf0 0x83")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf1 0xb5")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf2 0x09")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf3 0x00")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf4 0xa0")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf5 0x00")
		-- 16kHz - PAL
		elseif(ifo.freq == 16000 and ifo.format == "576i") then 
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf0 0x07")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf1 0x6b")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf2 0x13")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf3 0x00")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf4 0x40")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf5 0x01")
		-- 32kHz - PAL
		elseif(ifo.freq == 32000 and ifo.format == "576i") then 
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf0 0x0e")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf1 0xd6")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf2 0x26")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf3 0x00")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf4 0x80")
			board.execute("i2crw --sccb /dev/i2c-2 0x50 0xf5 0x02")
		else 
			print("Invalid kite audio format selected\n")
		end
                -- this should be default
                -- board.execute("i2crw --sccb /dev/i2c-2 0x50 0xdb 0xc2")

                -- audio asynr and aclkr are master
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0xdb 0xc1")
 	        board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 0")
           end,
            
            onboard = function(ifo)
          
	              if board._config.audio_analog_init ~= true then
	                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 7")
	                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 9")
	                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --set 0")
	            
	            
		            board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 7")
		            board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 8");
		            board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 9");
		            board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 0");
		            board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 1");
		            board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 2");
		            board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 3");
		            board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 15");
		            board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 14");
		            board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 15");
		            board.execute("i2cexp --bus /dev/i2c-0 --dev 0x44 --set 14");            
		            board._config.audio_analog_init = true
	            end
	            
	            local sample = ifo.freq * 256;
	            local oversample = ifo.freq * 64;
	            print("Audio clock settings: " .. sample .. " " .. oversample)
                    if sample <= 4096000 then
                       print("Warning: turn off audio of HDMI output")
                       board.execute("i2crw --sccb /dev/i2c-3 0x7a 0xa 0x51")
                    end

	            if board._config.Q_SYS_CFG_PLL3_CLOCK_HZ == nil then
	               sysctl0:configure({param=Q_SYS_CFG_PLL3_CLOCK_HZ, value=sample})
	               board._config.Q_SYS_CFG_PLL3_CLOCK_HZ = sample
	            else
	               assert(board._config.Q_SYS_CFG_PLL3_CLOCK_HZ == sample, 
	                      "Sample Frequency must be identical on both port")
	            end
	            if ifo.port == 0 then
	                sysctl0:configure({param=Q_SYS_CFG_AUD0_CLOCK_HZ, value=sample})
	            else -- check for port ID is made in the set function
	                sysctl0:configure({param=Q_SYS_CFG_AUD1_CLOCK_HZ, value=sample})
	            end
            
            end,
            hdmi = function(ifo)
	            local sample = ifo.freq * 256;

                    if board._config.Q_SYS_CFG_PLL3_CLOCK_HZ == nil then
                       sysctl0:configure({param=Q_SYS_CFG_PLL3_CLOCK_HZ, value=sample})
                       board._config.Q_SYS_CFG_PLL3_CLOCK_HZ = sample
                    else
                       assert(board._config.Q_SYS_CFG_PLL3_CLOCK_HZ == sample,
                              "Sample Frequency must be identical on both port")
                    end
                      --Aud0 clock must always be set for audio out to work (like in preview)
                    sysctl0:configure({param=Q_SYS_CFG_AUD0_CLOCK_HZ, value=sample})
  
                    board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 8"); 
		    board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 9");
                    board.execute("i2crw /dev/i2c-2 0x98 0x58 0x01");
            end,
        },
        
        -- END AUDIO INPUT
        --------------------------------------------------------------------------
        
        --------------------------------------------------------------------------
        -- VIDEO OUTPUT
        vout = {
        --===============================================================
            component = function(ifo) 
                board.wr_gpio("02", 0) 
                board.syscall("usleep 100000")
                board.wr_gpio("02", 1)
                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 7")
                if ifo.format == "480i" then
                    board.set_device("ADV739x", "--bus /dev/i2c-3 --dev 0x54 --config 'ntsc_component'")
                    board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 8")
                    board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 9")
                elseif ifo.format == "576i" then
                    board.set_device("ADV739x", "--bus /dev/i2c-3 --dev 0x54 --config 'pal_component'")
                    board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 8")
                    board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 9")
                elseif ifo.format == "1080i" or ifo.format == "1080i50" then
                    board.set_device("ADV739x", "--bus /dev/i2c-3 --dev 0x54 --config hd_1080i_component")
                    board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 8")
                    board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --set 9")
                elseif ifo.format == "720p" or ifo.format == "720p50" then
                    board.set_device("ADV739x", "--bus /dev/i2c-3 --dev 0x54 --config hd_720p_component")
                    board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 8")
                    board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --set 9")
                else
                    error(tostring(ifo.format) .. " format not supported on component") 
                end

            end,
            --===============================================================
            svideo = function(ifo) 
                -- by default vid23 sources video encoder    
                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --set 7")
                -- reset the video encoder
                board.wr_gpio("02", 0)
                board.syscall("usleep 100000")
                board.wr_gpio("02", 1) 
                if ifo.format == "480i" then
                    board.set_device("ADV739x", "--bus /dev/i2c-3 --dev 0x54 --config 'ntsc_svideo'")
                elseif ifo.format == "576i" then
                    board.set_device("ADV739x", "--bus /dev/i2c-3 --dev 0x54 --config 'pal_svideo'")
                else
                    error(tostring(ifo.format) .. " format not supported on svideo") 
                end
            end,
            --===============================================================
            composite = function(ifo)
                board._support.mg3500_evp.vout.svideo(ifo)
            end,

            --===============================================================         
            hdmi = function(ifo)                            
                board.wr_gpio("02", 0)
                board.syscall("usleep 100000")
                board.wr_gpio("02", 1)
                assert(ifo.external ~= 1, "External sync not supported on evp")
                board.set_device("AD9889", "--bus /dev/i2c-3 --dev 0x7A --format %s --input YUV --output RGB --style 2 --656", 
                            ifo.format)
            end,

        }, 
        -- END VIDEO OUTPUT
        --------------------------------------------------------------------------
        -- VIDEO INPUT
        -- Set up the input connector on evp
        -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 0")
        -- 0: video from Kite
        -- 1: video from EVP on board
        
        vin = {
            --===============================================================
            component = function(ifo)
                board.wr_gpio("46", 1)
                board.wr_gpio("46", 0)
                board.syscall("usleep 100000")
                board.wr_gpio("46", 1)
                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 0")
                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 1")
                board.set_device("AD9880", "--bus /dev/i2c-2 --dev 0x98 --invpwr --format %s --input YUV --output YUV --analog 0", 
                        ifo.format)
                --FIXME: We have to disable audio for now, to be investigated
                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 9")
            end,
            --===============================================================
            svideo = function(ifo)
                if ifo.port == 1 then
                    address = "0x42"
                else
                    address = "0x40"
                end

                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 0")
                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 1")

                if ifo.format == "480i" then
                    board.set_device("SAA7114", "--bus /dev/i2c-2 --dev "..address.." --ntsc_svideo")
                elseif ifo.format == "576i" then
                    board.set_device("SAA7114", "--bus /dev/i2c-2 --dev "..address.." --pal_svideo")
                else
                    error(tostring(ifo.format) .. " format not supported on svideo") 
                end
            end,
            --===============================================================
            composite = function(ifo)
                if ifo.port == 1 then
                    address = "0x42"
                else
                    address = "0x40"
                end

                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 0")
                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 1")

                if ifo.format == "480i" then
                    board.set_device("SAA7114", "--bus /dev/i2c-2 --dev "..address.." --ntsc_composite")
                elseif ifo.format == "576i" then
                    board.set_device("SAA7114", "--bus /dev/i2c-2 --dev "..address.." --pal_composite")
                else
                    error(tostring(ifo.format) .. " format not supported on svideo") 
                end
            end,                         
            --===============================================================
            hdmi = function(ifo)
                board.wr_gpio("46", 1)
                board.wr_gpio("46", 0)
                board.syscall("usleep 100000")
                board.wr_gpio("46", 1)
                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 0")
                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 1")               
                board.set_device("AD9880", "--bus /dev/i2c-2 --dev 0x98 --invpwr --format %s --input RGB --output YUV --digital", 
                        ifo.format)                 
            end,
            --===============================================================
            kite = function(ifo)

                -- The following is only executed once
                board.cfg.kiteReset()
                                
                -- Set up the input connector on evp
                board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 0")
		
                -- setup TW2864 video
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0x96 0xe6")

                -- Ch ID with the specific 656 sync code
                -- board.execute("i2crw --sccb /dev/i2c-2 0x50 0x9e 0x52")

                -- Ch ID with the specific horizontal blanking code
                -- board.execute("i2crw --sccb /dev/i2c-2 0x50 0x9e 0x62")

                -- Ch ID with the specific horizontal 656 sync & horizontal blanking code
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0x9e 0x72")

                -- Two channel time multiplexed on VD1 and VD2
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0xca 0x55")

                -- Set channel assignment
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0xcc 0xee")
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0xcd 0x44")

                -- Set CLKN to be inverted, and CLKP to be non-inverted.
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0xfb 0x0f")

                -- Enable output and both CLKPO and CLKNO, and set both to 27MHz
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0xfa 0x40")

                -- Setting brightness and contrast values to TW's recommendations.
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0x01 0x00")
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0x11 0x00")
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0x21 0x00")
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0x31 0x00")
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0x02 0x64")
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0x12 0x64")
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0x22 0x64")
                board.execute("i2crw --sccb /dev/i2c-2 0x50 0x32 0x64")

		if (ifo.format == "576i") then 
                    board.execute("i2crw --sccb /dev/i2c-2 0x50 0x08 0x17")
                    board.execute("i2crw --sccb /dev/i2c-2 0x50 0x18 0x17")
                    board.execute("i2crw --sccb /dev/i2c-2 0x50 0x28 0x17")
                    board.execute("i2crw --sccb /dev/i2c-2 0x50 0x38 0x17")
                end


                -- The rest of the techwell registers have their default values. 

                -- kite fpga setup
                -- Read the version. Olf FPGA version will fail the i2c read (and 
                -- attributed a version of 0) which doesn't support PAL
                
                local version = nil
                local tmpname= "/tmp/version.tmp"
                
                if board.syscall("i2crw --sccb /dev/i2c-2 0xc0 0xf8 2>&1 1>" .. tmpname ) == 0 then
                
	                local f= io.open( tmpname, OPEN_RT )
	                if f then
	                    str= f:read("*a")   -- read all at once        
	                    f:close()        
	                end
	                --os.remove(tmpname)                
	                version = tonumber(string.match(str, "0xf8:%s+(%w+)"))
                
                end
                
                version = version or 0
                                
                print(string.format("[KITE VERSION] 0x%x", version))
                
                if version == 0 then
                    assert(ifo.format ~= "576i", "PAL Not supported with this Kite FPGA version")
                else

		if (ifo.format == "576i") then 
                    board.execute("i2crw --sccb /dev/i2c-2 0x50 0x08 0x17")
                    board.execute("i2crw --sccb /dev/i2c-2 0x50 0x18 0x17")
                    board.execute("i2crw --sccb /dev/i2c-2 0x50 0x28 0x17")
                    board.execute("i2crw --sccb /dev/i2c-2 0x50 0x38 0x17")
                end
                
	                board.execute("i2crw --sccb /dev/i2c-2 0xc0 0x00 0x00")
		            board.execute("i2crw --sccb /dev/i2c-2 0xc0 0x01 0x22");
		            board.execute("i2crw --sccb /dev/i2c-2 0xc0 0x02 0x22");
		            board.execute("i2crw --sccb /dev/i2c-2 0xc0 0x03 0x00");
		            board.execute("i2crw --sccb /dev/i2c-2 0xc0 0x04 0x00");
		            board.execute("i2crw --sccb /dev/i2c-2 0xc0 0x05 0x00");
		            if ifo.format == "480i" or ifo.format == "kite" then
		               board.execute("i2crw --sccb /dev/i2c-2 0xc0 0x06 0x8a");
		               board.execute("i2crw --sccb /dev/i2c-2 0xc0 0x07 0x2c");
		               board.execute("i2crw --sccb /dev/i2c-2 0xc0 0x00 0x05");
		            elseif ifo.format == "576i" then
		               board.execute("i2crw --sccb /dev/i2c-2 0xc0 0x06 0x90");
		               board.execute("i2crw --sccb /dev/i2c-2 0xc0 0x07 0x30");
		               board.execute("i2crw --sccb /dev/i2c-2 0xc0 0x00 0x07");
		            else
		               error(tostring(ifo.format) .. " format not supported on kite")
		            end
		        end
            end,
            -----------------------------------------------------------------
        }, 
      -- END VIDEO INPUT
      --------------------------------------------------------------------------
   },
   -- END EVP SOC
   ----------------------------------------------------------------------------- 
-- DSX SOC
    mg3500_evp = {
        dev_root   = "/usr/bin", -- used in board.set_device
        gpio_root  = "/proc/driver/gpio/native", -- require by board.wr_gpio
        sysctlvout = function(ifo)
          if ifo.format == "720p" or ifo.format == "1080i" or ifo.format == "1080p" or ifo.format == "720p50"  or ifo.format == "1080i50" then
             if ifo.clockntsc == 0 then
               sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=74250000})
               -- sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_FLAGS,    value=Q_SYS_CFP_CLOCK_FLAG_INPUT + Q_SYS_CFP_CLOCK_FLAG_OUTPUT}); 
               sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value=74250000})
             else
               sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=74175824})
               -- sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_FLAGS,    value=Q_SYS_CFP_CLOCK_FLAG_INPUT + Q_SYS_CFP_CLOCK_FLAG_OUTPUT}); 
               sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value=74175824})
             end

             sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL, value=0x0010aac0})  
             -- sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL, value=0x0010aac0})  
             -- sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL, value=0x001295c0})  
             -- sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL, value=0x001095c0}) 
             -- sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL, value=0x001895c0})             
          else
             sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=27000000})
             sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value=27000000})
             sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL,             value=0x0012aa00})      
          end    
          sysctl0:configure({param=Q_SYS_CFG_VIDEO_INVERT_CONTROL, value=0x00000000})
          -- sysctl0:configure({param=Q_SYS_CFG_VIDEO_INVERT_CONTROL, value=0x00000004})
        end,
                   
        sysctlvin = function(ifo)
          if ifo.format == "720p" or ifo.format == "1080i" or ifo.format == "1080p" or ifo.format == "720p50" or ifo.format == "1080i50" or ifo.mode == "kite" then
             sysctl0:configure({param=Q_SYS_CFG_VIN0_CONTROL, value=0x00048000}) 
             sysctl0:configure({param=Q_SYS_CFG_VIN1_CONTROL, value=0x00000000})
          else
             if ifo.port == 1 then
                 sysctl0:configure({param=Q_SYS_CFG_VIN1_CONTROL,             value = 0x00049249})
             else
                 sysctl0:configure({param=Q_SYS_CFG_VIN0_CONTROL,             value = 0x00040000})
             end
          end
        end,

       -- boardInit is called by the detect() method once the board has been detected.
       boardInit = function(ifo)
        local h = assert(io.open(board.cfg.gpio_root, "r", "Failed to find gpio path"))
        h:close()
             
	    board.wr_gpio("46", 1)
	    board.wr_gpio("46", 0)
	    board.syscall("usleep 100000")
	    board.wr_gpio("46", 1)
      end,
     
             
      -- Kite reset shall be called the first time the board is initialized
      -- Enabling 54MHz clock to Kite does not do any harm even if there is an 
      -- oscillator on Kite board. 
      -- Since the clock is enabled there and reset pulse is already toggled, you
      -- do not need to toggle it again. 
      kiteReset = function()
      
        -- if board._config.kite_reset ~= true then
        
            -- This should be done only once
      
	        -- sysctl0:configure   {
	                                 -- param   =   Q_SYS_CFG_V0_PIXEL_CLOCK_FLAGS, 
	                                 -- value   =   (Q_SYS_CFP_CLOCK_FLAG_OUTPUT + Q_SYS_CFP_CLOCK_FLAG_INPUT)
	                            -- }
	
	        -- sysctl0:configure   {
	                                 -- param   =   Q_SYS_CFG_V0_PIXEL_CLOCK_HZ, 
	                                 -- value   =   54000000
	                            -- }
	        -- sysctl0:configure   {
	                                 -- param   =   Q_SYS_CFG_SERIALIO_DRIVESTRENGTH_1, 
	                                 -- value   =   0x000026c9
	                            -- }
	                    
	        -- board.wr_gpio("46", 1)
	        -- board.wr_gpio("46", 0)
	        -- board.syscall("usleep 100000")
	        -- board.wr_gpio("46", 1)
	        -- board._config.kite_reset = true
	        
        -- end      
      
      end,
      
        --------------------------------------------------------------------------
        -- AUDIO INPUT
        -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 7")
        -- 0: audio0 from Kite
        -- 1: audio0 from EVP on board
        -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 9")
        -- 0: audio1 from Kite
        -- 1: audio1 from EVP on board
        -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 0")
        -- 0: MG3500 becomes audio output clock slave (Kite)
        -- 1: MG3500 becomes audio output clock master (EVP)       
        
        ain = {
            
            onboard = function(ifo)
          
	              if board._config.audio_analog_init ~= true then
	                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 7")
	                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 9")
	                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --set 0")
	            
	            
		            -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 7")
		            -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 8");
		            -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 9");
		            -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 0");
		            -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 1");
		            -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 2");
		            -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 3");
		            -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 15");
		            -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 14");
		            -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 15");
		            -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x44 --set 14");
		            board._data.sys_rc = 256;
		            board.execute("/usr/local/bin/ppf w 90 0");   -- power off DAC/ADC second
		            board.execute("/usr/local/bin/ppf w 90 200"); -- power on ADC first
		            board.execute("/usr/local/bin/ppf w 90 8200"); -- then power on DAC second
		            board._data.sys_rc = 0;            
		            board._config.audio_analog_init = true
	            end
	            
	            local sample = ifo.freq * 256;
	            local oversample = ifo.freq * 64;
	            print("Audio clock settings: " .. sample .. " " .. oversample)
                    if sample <= 4096000 then
                       print("Warning: turn off audio of HDMI output")
                       board.execute("i2crw --sccb /dev/i2c-3 0x7a 0xa 0x51")
                    end

	            if board._config.Q_SYS_CFG_PLL3_CLOCK_HZ == nil then
	               sysctl0:configure({param=Q_SYS_CFG_PLL3_CLOCK_HZ, value=sample})
	               board._config.Q_SYS_CFG_PLL3_CLOCK_HZ = sample
	            else
	               assert(board._config.Q_SYS_CFG_PLL3_CLOCK_HZ == sample, 
	                      "Sample Frequency must be identical on both port")
	            end
	            if ifo.port == 0 then
	                sysctl0:configure({param=Q_SYS_CFG_AUD0_CLOCK_HZ, value=sample})
	            else -- check for port ID is made in the set function
	                sysctl0:configure({param=Q_SYS_CFG_AUD1_CLOCK_HZ, value=sample})
	            end
            
            end,
            hdmi = function(ifo)
	            local sample = ifo.freq * 256;

                    if board._config.Q_SYS_CFG_PLL3_CLOCK_HZ == nil then
                       sysctl0:configure({param=Q_SYS_CFG_PLL3_CLOCK_HZ, value=sample})
                       board._config.Q_SYS_CFG_PLL3_CLOCK_HZ = sample
                    else
                       assert(board._config.Q_SYS_CFG_PLL3_CLOCK_HZ == sample,
                              "Sample Frequency must be identical on both port")
                    end
                      --Aud0 clock must always be set for audio out to work (like in preview)
                    sysctl0:configure({param=Q_SYS_CFG_AUD0_CLOCK_HZ, value=sample})
  
                    -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 8"); 
		            -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 9");
                    -- board.execute("i2crw /dev/i2c-2 0x98 0x58 0x01");
            end,
        },
        
        -- END AUDIO INPUT
        --------------------------------------------------------------------------
        
        --------------------------------------------------------------------------
        -- VIDEO OUTPUT
        vout = {
        --===============================================================
            component = function(ifo) 
                -- board.wr_gpio("02", 0) 
                -- board.syscall("usleep 100000")
                -- board.wr_gpio("02", 1)
                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 7")
                -- if ifo.format == "480i" then
                    -- board.set_device("ADV739x", "--bus /dev/i2c-3 --dev 0x54 --config 'ntsc_component'")
                    -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 8")
                    -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 9")
                -- elseif ifo.format == "576i" then
                    -- board.set_device("ADV739x", "--bus /dev/i2c-3 --dev 0x54 --config 'pal_component'")
                    -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 8")
                    -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 9")
                -- elseif ifo.format == "1080i" or ifo.format == "1080i50" then
                    -- board.set_device("ADV739x", "--bus /dev/i2c-3 --dev 0x54 --config hd_1080i_component")
                    -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 8")
                    -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --set 9")
                -- elseif ifo.format == "720p" or ifo.format == "720p50" then
                    -- board.set_device("ADV739x", "--bus /dev/i2c-3 --dev 0x54 --config hd_720p_component")
                    -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --clear 8")
                    -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --set 9")
                -- else
                    -- error(tostring(ifo.format) .. " format not supported on component") 
                -- end

            end,
            --===============================================================
            svideo = function(ifo) 
                -- by default vid23 sources video encoder    
                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x4e --set 7")
                -- reset the video encoder
                -- board.wr_gpio("02", 0)
                -- board.syscall("usleep 100000")
                -- board.wr_gpio("02", 1) 
                -- if ifo.format == "480i" then
                    -- board.set_device("ADV739x", "--bus /dev/i2c-3 --dev 0x54 --config 'ntsc_svideo'")
                -- elseif ifo.format == "576i" then
                    -- board.set_device("ADV739x", "--bus /dev/i2c-3 --dev 0x54 --config 'pal_svideo'")
                -- else
                    -- error(tostring(ifo.format) .. " format not supported on svideo") 
                -- end
            end,
            --===============================================================
            composite = function(ifo)
                -- board._support.mg3500_evp.vout.svideo(ifo)
            end,

            --===============================================================         
            hdmi = function(ifo)                            
                -- board.wr_gpio("02", 0)
                -- board.syscall("usleep 100000")
                -- board.wr_gpio("02", 1)
                -- assert(ifo.external ~= 1, "External sync not supported on evp")
                -- board.set_device("AD9889", "--bus /dev/i2c-3 --dev 0x7A --format %s --input YUV --output RGB --style 2 --656", 
                          --  ifo.format)
            end,

        }, 
        -- END VIDEO OUTPUT
        --------------------------------------------------------------------------
        -- VIDEO INPUT
        -- Set up the input connector on evp
        -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 0")
        -- 0: video from Kite
        -- 1: video from EVP on board
        
        vin = {
            --===============================================================
            component = function(ifo)
                -- board.wr_gpio("46", 1)
                -- board.wr_gpio("46", 0)
                -- board.syscall("usleep 100000")
                -- board.wr_gpio("46", 1)
                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 0")
                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 1")
                -- board.set_device("AD9880", "--bus /dev/i2c-2 --dev 0x98 --invpwr --format %s --input YUV --output YUV --analog 0", 
                        -- ifo.format)
                --FIXME: We have to disable audio for now, to be investigated
                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 9")
            end,
            --===============================================================
            svideo = function(ifo)
                -- if ifo.port == 1 then
                    -- address = "0x42"
                -- else
                    -- address = "0x40"
                -- end

                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 0")
                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 1")

                -- if ifo.format == "480i" then
                    -- board.set_device("SAA7114", "--bus /dev/i2c-2 --dev "..address.." --ntsc_svideo")
                -- elseif ifo.format == "576i" then
                    -- board.set_device("SAA7114", "--bus /dev/i2c-2 --dev "..address.." --pal_svideo")
                -- else
                    -- error(tostring(ifo.format) .. " format not supported on svideo") 
                -- end
            end,
            --===============================================================
            composite = function(ifo)
                -- if ifo.port == 1 then
                    -- address = "0x42"
                -- else
                    -- address = "0x40"
                -- end

                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 0")
                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 1")

                -- if ifo.format == "480i" then
                    -- board.set_device("SAA7114", "--bus /dev/i2c-2 --dev "..address.." --ntsc_composite")
                -- elseif ifo.format == "576i" then
                    -- board.set_device("SAA7114", "--bus /dev/i2c-2 --dev "..address.." --pal_composite")
                -- else
                    -- error(tostring(ifo.format) .. " format not supported on svideo") 
                -- end
            end,                         
            --===============================================================
            hdmi = function(ifo)
                -- board.wr_gpio("46", 1)
                -- board.wr_gpio("46", 0)
                -- board.syscall("usleep 100000")
                -- board.wr_gpio("46", 1)
                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --set 0")
                -- board.execute("i2cexp --bus /dev/i2c-0 --dev 0x40 --clear 1")               
                -- board.set_device("AD9880", "--bus /dev/i2c-2 --dev 0x98 --invpwr --format %s --input RGB --output YUV --digital", 
                        -- ifo.format)                 
            end,
            --===============================================================
            kite = function(ifo)

            end,
            -----------------------------------------------------------------
        }, 
      -- END VIDEO INPUT
      --------------------------------------------------------------------------
   },
   -- END DSX SOC
   ----------------------------------------------------------------------------- 
   -- mobicam2 SOC
   mg3500_mobicam2 = {
        dev_root   = "/usr/bin", -- used in board.set_device
        gpio_root  = "/proc/driver/gpio/native", -- require by board.wr_gpio
        sysctlvout = function(ifo)
        --sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=27000000})
          sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_FLAGS,     value=Q_SYS_CFP_CLOCK_FLAG_OUTPUT}); 
          sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value=27000000})
          sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL,             value=0x001200aa})         
        --sysctl0:configure({param=Q_SYS_CFG_VIDEO_INVERT_CONTROL,   value=0x00000000})
        end,
                   
        sysctlvin = function(ifo)
             sysctl0:configure({param=Q_SYS_CFG_VIN0_CONTROL, value=0x00041249}) 
             sysctl0:configure({param=Q_SYS_CFG_VIN1_CONTROL, value=0x00000000})
        end,

        --------------------------------------------------------------------------
        -- AUDIO INPUT
        ain = {
              --===============================================================
              onboard = function(ifo)
                          -- Initialize the audio AK4633 audio codec
                          --os.execute( "/usr/bin/setAK4633 -v" )
                          board.set_device("AK4633", "-v -sf %d", ifo.freq);
                        end,
         },
         -- END AUDIO INPUT 

        -- boardInit is called by the detect() method once the board has been detected.
        boardInit = function(ifo) 
        end,

        --------------------------------------------------------------------------
        -- VIDEO OUTPUT
        vout = {
            composite = function(ifo)
              if ifo.format == "480i" then
                 board.execute( "/usr/bin/setSAA7121 --ntsc --bus /dev/i2c-2")
              else
                 board.execute( "/usr/bin/setSAA7121 --pal --bus /dev/i2c-2")
              end
            end,
        }, 
        -- END VIDEO OUTPUT
        --------------------------------------------------------------------------
       -- VIDEO INPUT          
       vin = {
            sensor = function(ifo)
            board.syscall("usleep 300000") 	
            --reset sensor by GPIO_2_13
            os.execute( "echo 'function 53 1' > /sys/bus/platform/drivers/m_ioctrl/gpio")
            board.wr_gpio("53", 0)
            board.wr_gpio("53", 1)          
            board.syscall("usleep 300000") 
        
            if     ifo.format == "720p25" then
                -- provide 24MHz clock to sensor
                sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=24000000})
                sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_FLAGS,     value=Q_SYS_CFP_CLOCK_FLAG_INPUT + Q_SYS_CFP_CLOCK_FLAG_OUTPUT}); 
                sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_HZ,        value=24000000})

                board.execute( "i2cprog --dev /dev/i2c-0 /var/mobicam2/sensorinit_720p25.i2c" )
            elseif ifo.format == "720p30" then
                -- provide 12MHz clock to sensor (can also use 24MHz if we make corresponding change in sensor script)
                                                        
                sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=12000000})
                sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_FLAGS,     value=Q_SYS_CFP_CLOCK_FLAG_INPUT + Q_SYS_CFP_CLOCK_FLAG_OUTPUT}); 
                sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_HZ,        value=12000000})

                board.execute( "i2cprog --dev /dev/i2c-0 /var/mobicam2/sensorinit_720p30.i2c" )
            elseif ifo.format == "uxgap15" then
                -- provide 12MHz clock to sensor (can also use 24MHz if we make corresponding change in sensor script)
                sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=12000000})
                sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_FLAGS,     value=Q_SYS_CFP_CLOCK_FLAG_INPUT + Q_SYS_CFP_CLOCK_FLAG_OUTPUT}); 
                sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_HZ,        value=12000000})

                board.execute( "i2cprog --dev /dev/i2c-0 /var/mobicam2/sensorinit_uxgap15.i2c" )
            else
                error(tostring(ifo.format) .. " format not supported on the sensor")
            end
        end,
      }, 
      -- END VIDEO INPUT
   --------------------------------------------------------------------------
   },
   -- END mobicam2 SOC
   -----------------------------------------------------------------------------
   -----------------------------------------------------------------------------
   -- mobicam3 SOC
   mg3500_mobicam3 = {
        dev_root   = "/usr/bin", -- used in board.set_device
        gpio_root  = "/proc/driver/gpio/native", -- require by board.wr_gpio
        sysctlvout = function(ifo)
        --sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=27000000})
          sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_FLAGS,     value=Q_SYS_CFP_CLOCK_FLAG_OUTPUT}); 
          sysctl0:configure({param=Q_SYS_CFG_V2_PIXEL_CLOCK_HZ,        value=27000000})
          sysctl0:configure({param=Q_SYS_CFG_VOUT_CONTROL,             value=0x001200aa})         
        --sysctl0:configure({param=Q_SYS_CFG_VIDEO_INVERT_CONTROL,   value=0x00000000})
        end,
                   
        sysctlvin = function(ifo)
             sysctl0:configure({param=Q_SYS_CFG_VIN0_CONTROL, value=0x00041249}) 
             sysctl0:configure({param=Q_SYS_CFG_VIN1_CONTROL, value=0x00000000})
        end,

        --------------------------------------------------------------------------
        -- AUDIO INPUT
        ain = {
              --===============================================================
              onboard = function(ifo)
                    -- MAX9860 MCLK is connected to AUD0 in boards without rework and can only support 48000 sampling rate.
                    -- MAX9860 MCLK is connected to AUD1 in boards with rework to support all sampling rates.
	            local sample = ifo.freq * 256;
                    local sample_port1;

                    if (ifo.freq == 48000) then
                      -- Set MAX9860 MCLK to 256*48000 = 12.288MHz so that we
                      -- can use the same software for boards without the rework
                      sample_port1 = sample; 
                    else
                      sample_port1 =  12000000; --Should match with PLL2 used for sensor
                    end
	            local oversample = ifo.freq * 64;
	            print("Audio clock settings: " .. sample .. " " .. oversample)
	            if board._config.Q_SYS_CFG_PLL3_CLOCK_HZ == nil then
	               sysctl0:configure({param=Q_SYS_CFG_PLL3_CLOCK_HZ, value=sample})
	               board._config.Q_SYS_CFG_PLL3_CLOCK_HZ = sample
	            else
	               assert(board._config.Q_SYS_CFG_PLL3_CLOCK_HZ == sample, 
	                      "Sample Frequency must be identical on both port")
	            end

                    sysctl0:configure({param=Q_SYS_CFG_AUD1_CLOCK_HZ, value=sample_port1});
	            if ifo.port == 0 then
	                sysctl0:configure({param=Q_SYS_CFG_AUD0_CLOCK_HZ, value=sample})
	            else -- check for port ID is made in the set function
	               assert(ifo.port == 0, 
	                      "Cannot use port1")
	            end
                        print("configuring MAX9860")
                        board.set_device("MAX9860", "--bus /dev/i2c-2 --dev 0x20 -sf %d -mclk %d", ifo.freq, sample_port1);
                        end,
         },
         -- END AUDIO INPUT 

        -- boardInit is called by the detect() method once the board has been detected.
        boardInit = function(ifo) 
        end,

        --------------------------------------------------------------------------
        -- VIDEO OUTPUT
        vout = {
            composite = function(ifo)
                if ifo.format == "480i" then
                    board.set_device("ADV739x", "--bus /dev/i2c-2 --dev 0x54 --config 'ntsc_svideo'")
                elseif ifo.format == "576i" then
                    board.set_device("ADV739x", "--bus /dev/i2c-2 --dev 0x54 --config 'pal_svideo'")
                else
                    error(tostring(ifo.format) .. " format not supported on svideo")
                end
            end,
        }, 
        -- END VIDEO OUTPUT
        --------------------------------------------------------------------------
        -- VIDEO INPUT          
        vin = {
            sensor = function(ifo)
              --[[
                Use the following reset sequence as mentioned in section "Hard Reset Sequence"
                of MT9D131 sensor databook:
                1. Wait for all power supplies to be stable and within specification.
                2. Supply the sensor with an input clock.
                3. Assert RESET# (active LOW) for at least 1µs.
                4. De-assert RESET# (input clock must be running).
                5. Wait 24 clock cycles before using the two-wire serial interface.
              --]]

              -- GPIO_2_14 is connected to sensor reset
              -- Set pin to GPIO mode
              -- The GPIO number used in Linux is found out from the command:
              -- cat /proc/driver/gpio/native/status2
              -- This returns the following after first time power up:
              -- 14,gpio054: d i,0 0,0 

              os.execute( "echo 'function 54 1' > /sys/bus/platform/drivers/m_ioctrl/gpio")
              os.execute( "echo 'pullup 54' > /sys/bus/platform/drivers/m_ioctrl/gpio")

              if     ifo.format == "720p25" then
                -- provide 12MHz clock to sensor  (can also use 24MHz if we make corresponding change in sensor script)
                sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=12000000})
                sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_FLAGS,     value=Q_SYS_CFP_CLOCK_FLAG_INPUT + Q_SYS_CFP_CLOCK_FLAG_OUTPUT}); 
                sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_HZ,        value=12000000})

                board.syscall("usleep 300000") 	
                --reset sensor by GPIO_2_14
                board.wr_gpio("54", 0)
                board.syscall("usleep 1") 
                board.wr_gpio("54", 1)          
                board.syscall("usleep 300000") 	

                board.execute( "i2cprog --dev /dev/i2c-0 /var/mobicam3/sensorinit_720p25.i2c" )
              elseif ifo.format == "720p30" then
                -- provide 12MHz clock to sensor (can also use 24MHz if we make corresponding change in sensor script)
                                                        
                sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=12000000})
                sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_FLAGS,     value=Q_SYS_CFP_CLOCK_FLAG_INPUT + Q_SYS_CFP_CLOCK_FLAG_OUTPUT}); 
                sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_HZ,        value=12000000})

                board.syscall("usleep 300000") 	
                --reset sensor by GPIO_2_14
                board.wr_gpio("54", 0)
                board.syscall("usleep 1") 
                board.wr_gpio("54", 1)          
                board.syscall("usleep 300000") 	
                board.execute( "i2cprog --dev /dev/i2c-0 /var/mobicam3/sensorinit_720p30.i2c" )
              elseif ifo.format == "uxgap15" then
                -- provide 12MHz clock to sensor (can also use 24MHz if we make corresponding change in sensor script)
                sysctl0:configure({param=Q_SYS_CFG_PLL2_CLOCK_HZ,            value=12000000})
                sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_FLAGS,     value=Q_SYS_CFP_CLOCK_FLAG_INPUT + Q_SYS_CFP_CLOCK_FLAG_OUTPUT}); 
                sysctl0:configure({param=Q_SYS_CFG_V1_PIXEL_CLOCK_HZ,        value=12000000})
                board.syscall("usleep 300000") 	
                --reset sensor by GPIO_2_14
                board.wr_gpio("54", 0)
                board.syscall("usleep 1") 
                board.wr_gpio("54", 1)          
                board.syscall("usleep 300000") 	

                board.execute( "i2cprog --dev /dev/i2c-0 /var/mobicam3/sensorinit_uxgap15.i2c" )
              else
                error(tostring(ifo.format) .. " format not supported on the sensor")
              end
        end,
      }, 
      -- END VIDEO INPUT
   --------------------------------------------------------------------------
   },
   -- END mobicam3 SOC
   -----------------------------------------------------------------------------
   -- Fake Board
   -- This for testing purposes on an i686
   i686_workstation = {
      sysctlvout = function(ifo) end,
      sysctlvin = function(ifo) end,
      --------------------------------------------------------------------------
      -- AUDIO OUTPUT
      ain = {
              --===============================================================
              onboard = function(ifo)
                        end,
       },       
      -- END AUDIO OUTPUT
      --------------------------------------------------------------------------
      
      --------------------------------------------------------------------------
      -- VIDEO OUTPUT
      
      --------------------------------------------------------------------------
         -- VIDEO OUTPUT
      vout = {
               --===============================================================
               component = function(ifo) print("vout comp i686_worstation")end,
               --===============================================================
               svideo =    function(ifo) print("vout svideo i686_worstation")end,
               --===============================================================
               composite = function(ifo) print("vout composite i686_worstation")end,
               --===============================================================         
               hdmi      = function(ifo) print("vout hdmi i686_worstation")end,
               --===============================================================
               sdi     =   function(ifo) print("vout sdi i686_worstation")end,
      }, 
      -- END VIDEO OUTPUT
      --------------------------------------------------------------------------
         -- VIDEO INPUT          
      vin = {
               --===============================================================
               component = function(ifo) print("vin comp i686_worstation")end,
               --===============================================================
               svideo =    function(ifo) print("vin svideo i686_worstation")end,
               --===============================================================
               composite = function(ifo) print("vin composite i686_worstation")end,
               --===============================================================
               hdmi      = function(ifo) print("vin hdmi i686_worstation")end,
               --===============================================================
               sdi   =     function(ifo) print("vin sdi i686_worstation")end,
               -----------------------------------------------------------------
      }, 
      -- END VIDEO INPUT
      --------------------------------------------------------------------------
             
   },
   -- END Fake Board
   -----------------------------------------------------------------------------  
}
----------------------------------------------------------------------------------
--[[    Board identifier creation

        /etc/mobilygen/board
        merlin-mips    : merlinbup-mg3500-none
        merlin-proto   : merlinproto-mg3500-fpgaexp
        evp            : arm-merlin
        
        /proc/driver/board/type
        evp            : mg3500_evp
        merlin-arm     : merlinbup
        merlin-mips    : non existent 
        merlin-proto   : merlinproto
        mobicam2       : mobicam2
        mobicam3       : mobicam3
        
--]]

--------------------------------------------------------------------------------
--- Detects the board and host type.
--
-- This method detects the board and host type by reading the information from
-- /etc/mobilygen/board and /proc/driver/board/type. This function will assert
-- if it cannot resolve the board type and host or if there is no corresponding
-- entry in the board._support table.
--
-- @see board.getType
-- @see board.getHost
function board.detect()

   board._ifo.type = "unknown"
   board._ifo.host = "unknown"
   local ifo
   
   if _G.QA_BOARD_OBJECT_I686_WORKSTATION == true then
    board._ifo.type = "workstation"
    board._ifo.host = "i686"
   else   
   
       local handle = io.open("/etc/mobilygen/board", "r")
       if not handle then
           handle = assert(io.open("/etc/sysconfig/board", "r"), "Failed to open /etc/sysconfig/board")
       end

       for line in handle:lines() do
           local tmp = line:lower():match("(%w+)")
           if tmp ~= nil then
              ifo = line
              board._ifo.type = tmp:lower()
           end
           if board._ifo.type == "arm" and line == "arm-merlin" then
              board._ifo.type = "arm-merlin"
           end
       end
    
       if board._ifo.type:find("^merlin") then
          local tmp = ifo:match("(%w+)$"):lower()
          if tmp == "none" or tmp == "fpgaexp" then
             board._ifo.host = "mips"
          end
       elseif board._ifo.type == "arm-merlin" then
          local handle = assert(io.open("/proc/driver/board/type", "r"), "Failed to open /proc/driver/board/type")
          board._ifo.host = "mg3500"
          for line in handle:lines() do
             if line:lower() == "merlinbup" then
                 board._ifo.type = "merlinbup"
             elseif line:lower() == "merlinproto" then
                 board._ifo.type = "merlinproto"
             elseif line:lower() == "mobicam2" then
                   board._ifo.type = "mobicam2"                   
             elseif line:lower() == "mobicam3" then
                   board._ifo.type = "mobicam3"                   
             else
                   local tmp = line:lower():match("mg3500_(%w+)") 
                 if tmp ~= nil then
                    board._ifo.type = tmp         
                 end
             end 
          end
       else
          error("Unknown board type " .. tostring(board._ifo.type))
       end      
   end
   assert(board._ifo.type ~= "unknown", "Failed to identify the board type")
   assert(board._ifo.host ~= "unknown", "Failed to identify the host")
    
   print(string.format("[BOARD TYPE: %s] [HOST: %s]\n", board._ifo.type, board._ifo.host))
   board._ifo.key  = board._ifo.host .. "_" .. board._ifo.type
    
   assert(board._support[board._ifo.key], "Unsupported board " ..  tostring(board._ifo.key))
   board.cfg = board._support[board._ifo.key]

   -- If some settings are specific to a board and must be done after the board detect, 
   -- the function specific to that board is called here
   if type(board.cfg.boardInit) == "function" then
	board.cfg.boardInit(board._ifo)
   end
	
   
end

--------------------------------------------------------------------------------
--                             PUBLIC METHODS
--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
---Returns the board type
-- @see board.detect
-- @return a string identifying the board type as detected in board.detect()
--
function board.getType() return board._ifo.type end

-------------------------------------------------------------------------------
---Returns the host type
-- @see board.detect
-- @return a string identifying the host type as detected in board.detect()
--
function board.getHost() return board._ifo.host end

-------------------------------------------------------------------------------
--- Returns the configuration paramaters of the last setVideoOutput call
function board.getVideoOutputConfig(port)
    if (type(port) ~= "number") then return nil end
    return board._data.vout[args.port]
end

-------------------------------------------------------------------------------
--- Returns the configuration paramaters of the last setVideoInput call
function board.getVideoInputConfig(port)
    if (type(port) ~= "number") then return nil end
    return board._data.vin[args.port]
end

-------------------------------------------------------------------------------
--- Configures the video output.
--
-- This method configures the board video output as well as internal settings
-- in the system control object. This is a generic entry point. Whether a 
-- particular mode is supported depends on the board capabilities.
--
-- This method will assert if there is no corresponding mode on the board.
-- 
-- @param args a table which must have the following fields
-- @param port port number
-- @param format 480i (NTSC), 480p, 576i (PAL), 576p, 720p, 1080i, 720p50, 1080i50
-- @param mode svideo, composite, component, hdmi, sdi
-- @param clockntsc set to 1 for 29.97, 59.94 etc
-- @return nil (function will assert if there is a problem)
--
-- @usage board.setVideoOutput({ port = 0, format = "720p",  mode = "hdmi"})
--
-- @see board.getVideoOutputConfig
function board.setVideoOutput(args)

   local usage = "Usage: board.setVideoOutput({port=[port number], format=['480i'|'480p'|'576i'|'576p'|'720p'|'1080i'|'1080p'|'720p50'|'1080i50'], mode=['svideo'|'hdmi'|'component'] [, clockntsc=[0 | 1]]})"

   if args["clockntsc"] == nil then
     args["clockntsc"] = 1
   end

   assert(type(args)           == "table",  usage)
   assert(type(args.mode)      == "string", usage)
   assert(type(args.format)    == "string", usage)
   assert(type(args.port)      == "number", usage)
   assert(type(args.clockntsc) == "number", usage)

   board._data.vout[args.port] = args
   board._data.vout[args.port].external = 0
   
   if args["sync"] == "external" then
        board._data.vout[args.port].external = 1
   end
   
   args.mode:lower()
   args.format:lower()

   assert( args.format == "ntsc" or args.format == "pal" or 
           args.format == "480i" or args.format == "480p" or 
           args.format == "576i" or args.format == "576p" or 
           args.format == "720p" or args.format == "1080i" or args.format == "1080p" or args.format == "720p50" or args.format == "1080i50", 
           "Invalid Format " .. tostring(args.format)) 
           
   if args.format == "pal" then args.format = "576i" end        
   if args.format == "ntsc" then args.format = "480i" end
   
   assert(board.cfg.vout[args.mode], "Invalid mode " .. tostring(args.mode))   
   board.cfg.sysctlvout(args)
   board.cfg.vout[args.mode](args)
    -- 0: sdi out from sdi in
    -- 1: sdi out from VPU

end

-------------------------------------------------------------------------------
--- Configures the video input.
--
-- This method configures the board video input as well as internal settings
-- in the system control object. This is a generic entry point. Whether a 
-- particular mode is supported depends on the board capabilities.
--
-- This method will assert if there is no corresponding mode on the board.
-- 
-- @param args a table which must have the following fields
-- @param port port number
-- @param format 480i (NTSC), 480p, 576i (PAL), 576p, 720p, 1080i, 1080p, 720p50, 1080i50
-- @param mode svideo, composite, component, hdmi, sdi
-- @return nil (function will assert if there is a problem)
--
-- @usage board.setVideoInput({ port = 0, format = "720p",  mode = "hdmi"})
-- @see board.getVideoInputConfig
function board.setVideoInput(args)

    local usage = "Usage: board.setVideoInput({port=[port number], format=['480i'|'480p'|'576i'|'576p'|'720p'|'1080i'|'1080p'|'720p50'|'1080i50'], mode=['hdmi'|'component'|'kite']})"
    
    assert(type(args)          == "table", usage)
    assert(type(args.mode)      == "string", usage)
    assert(type(args.format) == "string", usage)
    assert(type(args.port)      == "number", usage)
    
    board._data.vin[args.port] = args
    
    args.mode:lower()
    args.format:lower()

    assert( args.port == 0 or args.port == 1,
             "Invalid Port " .. tostring(args.port))

   
    assert( args.format == "ntsc" or args.format == "pal" or
            args.format == "480i" or args.format == "480p" or
            args.format == "576i" or args.format == "576p" or
            args.format == "720p25" or args.format == "720p30" or  --for mobicam
            args.format == "uxgap15" or args.format == "other" or  --for mobicam
            args.format == "720p" or args.format == "1080i" or args.format == "1080p" or args.format == "720p50" or args.format == "1080i50",
             "Invalid Format " .. tostring(args.format))  
            
   if args.format == "pal" then args.format = "576i" end        
   if args.format == "ntsc" then args.format = "480i" end
   
   assert(board.cfg.vin[args.mode], "Invalid mode " .. tostring(args.mode))  
   print( board.cfg.vin[args.mode] )
   board.cfg.sysctlvin(args)      
   board.cfg.vin[args.mode](args)  
    
end

--------------------------------------------------------------------------------
--- Configures the audio sample frequency and DAC/ADC.
--
-- Hardcoded to 256FS
--
-- @param args table containing the following fields
-- @param port port number
-- @param freq sample frequency 16[000], 24[000] 32[000] 44[.1|100] 48[000]
-- @param mode onboard or kite
--
-- @return nil
--------------------------------------------------------------------------------
function board.setAudioInput(args)

    local usage = "Usage: board.setAudioSampleFreq({port=[port number], freq=[16000|24000|32000|44100|48000], mode=[onboard|kite]})"

    if args.mode == nil then args.mode = "onboard" end    
    assert(type(args)          == "table", usage)
    assert(type(args.freq)      == "number", usage)
    assert(type(args.port)      == "number", usage)
    assert(type(args.mode)      == "string", usage)
    
    if args.freq < 100 then
       args.freq = args.freq * 1000
    end
    
    assert( args.port == 0 or args.port == 1,
             "Invalid Port " .. tostring(args.port))
    
    board._data.audio[args.port] = args
    board._data.ain[args.port] = args
    
    -- If both ports are being set, make sure that they are set to 
    -- the same freqency.    
    if board._data.audio[0] and board._data.audio[1] then
        assert(board._data.audio[0].freq == board._data.audio[1].freq, "Both port must be set at the same frequency")  
    end

    assert( args.freq == 8000 or args.freq == 16000 or args.freq == 24000 or
            args.freq == 32000 or args.freq == 44100 or
            args.freq == 48000, "Invalid Sample Frequency " .. tostring(args.freq))

    assert(board.cfg.ain[args.mode], "Invalid mode " .. tostring(args.mode))
    board.cfg.ain[args.mode](args)   

end


--------------------------------------------------------------------------------
--- Configures the audio sample frequency and DAC/ADC.
--
-- Hardcoded to 256FS
--
-- @param args table containing the following fields
-- @param port port number
-- @param freq sample frequency 16[000], 24[000] 32[000] 44[.1|100] 48[000]
--
-- @return nilboard.cfg.ain
--------------------------------------------------------------------------------
function board.setAudioSampleFreq(args)
    board.setAudioInput(args)
end

--------------------------------------------------------------------------------
--                             PRIVATE
--------------------------------------------------------------------------------
board.syscall = os.execute -- Please keep - for proxy object use



-------------------------------------------------------------------------------
--- Sets a gpio.
--
-- This method shall be used to set a gpio. The path is resolved based on the 
-- outcome of the board detect (see gpio_root in the board._support table)
-- 
-- @usage board.wr_gpio("03", 1)
--
-- @param gpio gpio id (string)
-- @param value value to set  
function board.wr_gpio(gpio, value)
    local gpio = string.format("%s/%s", board.cfg.gpio_root, tostring(gpio))
    -- print("[board object] ", gpio, value)
    local f = assert(io.open(gpio, "w"), "Failed to open " .. gpio .. " write")
    f:write(value)
    f:close()
     
    local f = assert(io.open(gpio, "r"), "Failed to open " .. gpio .. " read")
    local rval = tonumber(f:read("*a"))
    assert(rval == value, "Failed to write gpio " .. gpio .. "current value is " .. tostring(rval))
    f:close() 
end

-------------------------------------------------------------------------------
--- Configures a board device.
--
-- This method shall be used to configure a device on the board. The path to 
-- the executable is resolved based on the outcome of the board detection 
-- procedure (see dev_root in the board._support table)
-- 
-- @usage board.set_device("SAA7114", "--bus /dev/i2c-2 --dev 0x42 --ntsc_svideo")
--
-- @param device the device name without the set
-- @param ... arguments (same as printf)
-- 
function board.set_device(device, ...)
    assert(board.cfg.dev_root, "no base path for device config utils")
    local arguments = ""
    if arg.n ~= 0 then
        arguments = string.format(unpack(arg))
    end
    board.execute("%s/set%s %s", board.cfg.dev_root, device, arguments)
end


-------------------------------------------------------------------------------
--- Executes a system call.
--
-- This method executes a system call. In addition to the standard os.execute,
-- it prints the command to be executed on the standard output, and assert of 
-- the system call fails
--
-- @param fmt format string
-- @param ... arguments (same as printf)
--
-- @return rc, str return code and stdout, stderr command output
-- 
function board.execute(fmt, ...)
    local tmpname= "/tmp/uname.tmp"
    local str= nil
    local cmd = string.format(fmt, unpack(arg))
    -- print("[board object] ", cmd)
    local rc= board.syscall( cmd )
    --[[
    local rc= board.syscall( cmd .. " 2>&1 1>" .. tmpname )
    local f= io.open( tmpname, OPEN_RT )
    if f then
        str= f:read("*a")   -- read all at once        
        f:close()        
    end
    --]]
    assert(rc == board._data.sys_rc, "system call: " .. cmd .. "\n FAILED\n" .. tostring(rc) .. "\n")
    os.remove(tmpname)
    
    -- Remove terminating linefeed, if any (eases up one-line analysis)
    if str and string.sub( str, -1 ) == '\n' then
            str= string.sub( str, 1, -2 )
    end
    return rc, str
 end
 
-- This must be the last statement of the file
board.detect() -- detect the board type


--<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
-- PUT CODE BEFORE 
--<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
-- Temporary global variable to preserve the local variable board when existing 
-- the block - DO NOT REMOVE
__board = board
end              -- closing end for the do block
--Global variable board points to local reference of the board object.
board = __board
__board = nil

-- -----------------------------------------------------------------------------
-- vi: set shiftwidth=4 tabstop=8 softtabstop=4 expandtab nosmarttab:
-- -----------------------------------------------------------------------------
