#!/bin/sh

## simple script to run object tracking with DirectFB support.
## Parameters that can be passed to dfbobjtrack.so:
##      show_events     -- show all events with dimensions
##      tag_rect        -- tag each rectangle with a number
##      show_frame_event-- show the one and only frame event 
##      rect_thick      -- thickness of rectangles around moving
##                         objects, (0=solid rectangles, [1..8]=
##                         walls are 1..8 pixels thick)
##      reg_delay       -- wait number of seconds before registering
##                         callback.
##
##  All parameters that take arguments are of the form, <parameter>=<arg>
##
##  Usage: see usage() below
##

#-------------------------------------------------------------------------------
# The name of this script as invoked.
#-------------------------------------------------------------------------------
ME=`basename $0`

#-------------------------------------------------------------------------------
# Display an usage message to standard error.
#-------------------------------------------------------------------------------

usage()
{
    1>&2 cat << END_USAGE_MSG
Usage: ${ME} [options]

Shell script to run dvr script with object tracking support using DirectFB.
Since many things need to be set up in addition to running the lua
script, it is much simplier to just use this script instead of calling
the lua scripts directly.  Options are:

-d X    Specify object tracking display (1-4); only one out of the 4 outputs can
        show object tracking.
        Default is 1.

-a X    Specify the analytic streaming option (1 or 2) as documented 
        in Lua script. If enabled, analytic streaming will be done on
        all channels enabled for object tracking.  Default is 0.

-mgappdir X    Specify the mgapp directory if not using from /usr/bin.
               Should be the directory above the bin directory.

Examples:

    Object tracking displayed for first channel and no analytic streaming
    ${ME} 

    Object tracking displayed for second channel and no analytic streaming
    ${ME} -d 2

    Object tracking displayed for first channel and also analytic streaming
    in RTP metadata
    ${ME} -a 1
    
    Object tracking displayed for first channel and also analytic streaming
    in private SEI message
    ${ME} -a 2


END_USAGE_MSG
}

#-------------------------------------------------------------------------------
# Parse all command line argvs until the first one that does not begin
# with the '-' character.
#-------------------------------------------------------------------------------

# defaults
CHANNEL=1
ENABLE_ANALYTICS_STREAMING=0
MGAPPDIR=

while [ $# -ge 1 ] ; do
    case "${1}" in
    -h | --h | -help | --help)
        usage
        shift
        exit 0
        ;;
    -d)
        shift
        CHANNEL=${1}
        shift
        ;;
    -a)
        shift
        ENABLE_ANALYTICS_STREAMING=${1}
        shift
        ;;
    -mgappdir)
        shift
        MGAPPDIR=${1}
        shift
        ;;
    -*)
        1>&2 echo "${ME}: Unsupported option: ${1}"
        usage
        shift
        exit 1
        ;;
    *)
        break
        ;;
    esac
done

if [ "X${MGAPPDIR}" != "X" ]
then
    echo "Setting LD_LIBRARY_PATH"
    export LD_LIBRARY_PATH=$MGAPPDIR/lib:$MGAPPDIR/lib/plugins:/usr/lib:/usr/lib/plugins:${LD_LIBRARY_PATH}
fi

#-------------------------------------------------------------------------------
# setup call to lua script
#-------------------------------------------------------------------------------

# set script to use and initial parameters
SCRIPT="-f 4d1enc_4cifenc_dvr.lua -f dvr.lua -e STREAMING_ENCCH=2; OBJECT_TRACKING=1 ; OBJECTTRACKING_DISP=${CHANNEL} ; ENABLE_ANALYTICS_STREAMING =${ENABLE_ANALYTICS_STREAMING};"

# set final run string, run lua script and exit
T_CFG=/tmp/mgs
trap "rm ${T_CFG} ; exit 0" SIGINT SIGKILL
echo "mobiapp.so mobiapp ${SCRIPT}" >> ${T_CFG}
echo "ui_lua.so ui_lua ${SCRIPT}" >> ${T_CFG}
echo "dfbobjtrack.so" >> ${T_CFG}
mgapp -c ${T_CFG}
rm ${T_CFG}

exit 0
